require "go.app.ClientApp"

require "OmokScene"
require "OmokProtocol"

class 'OmokClientApp' (ClientApp)

function OmokClientApp:__init() super()
	math.randomseed(os.time())

	self:LoadXML("Scene.xml", true)

	self.scene = OmokScene(1)
	self:ActivateScene(self.scene)
	
	self:SetupSoundObjects()

	-- record current time
	self.lastsec = self:GetCurrentSeconds()

	-- Init
	self:initHandler()
end

---------------------------------------------------------
-- Server Init
---------------------------------------------------------
function OmokClientApp:initHandler()
	-- 추가된 stone을 저장하는 변수(리셋시 사용)
	self.board = {}
	self.boardNum = 0
	self:SetMsgHandler(Evt_Chat, self.onChat)
	self:SetMsgHandler(Evt_Start, self.onStart)
	self:SetMsgHandler(Evt_Put, self.onPut)
end

---------------------------------------------------------
-- Game Handler
---------------------------------------------------------
function OmokClientApp:OnEnterGame(player)
	self.isIn = true
	self.name = "Player" .. player.no
	self.no = player.no

	if player.no == 1 then
		-- 첫번째 사용자인 경우 방장 시작버튼 할당
		self.scene.textPlayerName1:SetText(self.name)
		local btnStart = Button(nil, 200, 50)
		self.scene:AddChild(btnStart)
		btnStart:LoadButtonImage("images/start.png", 4, 1, nil, 0)
		btnStart:SetXYPos(730, 157)
		btnStart.MouseDown:AddHandler(self.OnMouseDownStart)
	else
		-- 두번째 사용자는 준비버튼 할당
		self.scene.textPlayerName2:SetText(self.name)
		local btnReady = Button(nil, 200, 50)
		self.scene:AddChild(btnReady)
		btnReady:LoadButtonImage("images/ready.png", 4, 1, nil, 0)
		btnReady:SetXYPos(730, 157)
		btnReady.MouseDown:AddHandler(self.OnMouseDownReady)
	end
end

function OmokClientApp:OnLeaveGame(player)

end

function OmokClientApp:OnDisconnected()

end

function OmokClientApp:OnEnterPlayer(player)
	print(player)
	if player.no == 1 then
		self.scene.textPlayerName1:SetText("player" .. player.no)
	else
		self.scene.textPlayerName2:SetText("player" .. player.no)
	end
end

function OmokClientApp:OnLeavePlayer(player)

end

function OmokClientApp:OnTimer(timerid)

end

function OmokClientApp:Update(seconds)
	self.currentScene:Update(seconds - self.lastsec)
	self.lastsec = seconds
end

---------------------------------------------------------
-- Network Handler
---------------------------------------------------------
-- 채팅메세지 수신
function OmokClientApp:onChat(server, msg)
	local chattext = msg:GetValue(Evt_Chat.key.chattext)
	self.scene.textlistChat:AddLine(chattext, UIConst.White)
	self.scene.textlistChat:SetDisplayLine(UIConst.VertAlignBottom, self.scene.textlistChat:GetLineCount() - 1)
end

-- 시작
function OmokClientApp:onStart(server, msg)
	print("시작")
	self:initBoard()
	self.scene.textlistChat:AddLine("Player1부터 시작합니다.", UIConst.White)
	self.scene.textlistChat:SetDisplayLine(UIConst.VertAlignBottom, self.scene.textlistChat:GetLineCount() - 1)
end

function OmokClientApp:onPut(server, msg)
	print("onPut")
	local stone = StaticImage()
	self.scene:AddChild(stone)
	self.board[self.boardNum] = stone
	self.boardNum = self.boardNum + 1

	local color = "black"
	if msg:GetValue(Evt_Put.key.color) == 2 then
		color = "white"
	end
	stone:LoadStaticImage("images/" .. color .. ".png", 1, 1, nil, 0)
	stone:SetXYPos(msg:GetValue(Evt_Put.key.x) * 50, msg:GetValue(Evt_Put.key.y) * 50)
end

---------------------------------------------------------
-- Event Handler
---------------------------------------------------------
--Ready버튼 클릭 시
function OmokClientApp:OnMouseDownReady(msg)
	local msgReady = Message(Evt_Ready)
	gameapp:SendToServer(msgReady)
end

--Start버튼 클릭 시
function OmokClientApp:OnMouseDownStart(msg)
	local msgStart = Message(Evt_Start)
	gameapp:SendToServer(msgStart)
end

---------------------------------------------------------
-- 기타
---------------------------------------------------------
function OmokClientApp:initBoard()
	for i=1, self.boardNum do
		self.scene:RemoveChild(self.board[i-1])
	end
	self.board = {}
	self.boardNum = 0
end

require "OmokSoundObject"